#!/bin/sh

# Fixups to the do_mmap_pgoff function
# One and only arg is path to kernel source

# This extraction is a little more complicated than the others
# since do_mmap_pgoff calls a lot of other functions, many of 
# which are local to mmap.c

# We extract the function from the relevant source file
# - rename to skas_do_mmap_pgoff
# - add first argument of struct *mm
# - remove line using current->mm

cat <<EOF

extern int sysctl_overcommit_memory;

extern struct vm_area_struct * find_vma_prepare(struct mm_struct * mm, unsigned long addr,
						struct vm_area_struct ** pprev,
						rb_node_t *** rb_link, rb_node_t ** rb_parent);

extern int vma_merge(struct mm_struct * mm, struct vm_area_struct * prev,
		     rb_node_t * rb_parent, unsigned long addr, unsigned long end, unsigned long vm_flags);

extern void __vma_link(struct mm_struct * mm, struct vm_area_struct * vma,  struct vm_area_struct * prev,
		       rb_node_t ** rb_link, rb_node_t * rb_parent);

/* The kernel has two definitions.  One is when you are doing serious kernel
memory debugging, and the following which does nothing */
#define validate_mm(mm) do { } while (0)

EOF

(

./extractfunction.sh \
    'static inline unsigned long calc_vm_flags' \
         < $1/mm/mmap.c 	

./extractfunction.sh \
    'static inline void vma_link' \
         < $1/mm/mmap.c 	

./extractfunction.sh \
   'unsigned long do_mmap_pgoff' \
         < $1/mm/mmap.c | \
sed 's/long do_mmap_pgoff(/long skas_do_mmap_pgoff(struct mm_struct *mm, /' | \
sed 's/.*current->mm.*//'    

) | \
sed 's/calc_vm_flags/skas_calc_vm_flags/'