/* $Id: main.c,v 1.8 2002/12/18 09:16:58 rogerb Exp $ */

/* 
   Do the initialisation and destruction here.  We hook in our own
   ptrace system call.  I will be going to hell for this - see
   http://www.uwsg.iu.edu/hypermail/linux/kernel/0210.0/1108.html

   We also get /proc/mm created by calling into proc_mm.c.  The
   functions in there keep track of module usage counts.
*/



#define __KERNEL__
#define MODULE
#include <linux/modversions.h>
#include <linux/module.h>

#include <linux/kernel.h>
#include <linux/syscall.h>
#include <asm/unistd.h>

#include "skasptrace.h"
#include "proc_mm.h"

MODULE_LICENSE("GPL");

EXPORT_NO_SYMBOLS;

sys_ptrace_func chained_sys_ptrace=sys_ptrace;

extern sys_ptrace_func sys_call_table[];

#include "checksym.h"

/* Initialize the module */
int init_module(const char *name, struct module *image)
{
  int ret=-EBUSY;

  /* Sanity check #1.  Does the kernel address for sys_listen
     match the one in the System.map we were compiled against? */
  if (SYSTEMMAP_SYSLISTEN!=(void*)sys_listen)
    {
      printk("SKAS - Seperate Kernel Address Space Module\n"
	     "This module has been compiled against a different\n"
	     "kernel than you are using.  You need to rebuild\n"
	     "the module.\n");
      return -EBUSY;
    }

  /* Sanity check #2.  Does the syscall point to sys_ptrace? */

  if (sys_call_table[__NR_ptrace]!=(sys_ptrace_func)sys_ptrace)
    {
      printk("SKAS - Seperate Kernel Address Space Module\n"
	     "The ptrace system call slot is not pointing at\n"
	     "the kernel implementation of ptrace.  This will\n"
	     "either be due to the kernel being different than\n"
	     "this module was compiled against, or because some\n"
	     "other module has claimed the system call.  In\n"
	     "either case, it is not safe for this module to load\n");
      return -EBUSY;
    }

  /* initialise /proc/mm */
  ret=make_proc_mm();
  if (ret!=0)
    return ret;

  chained_sys_ptrace=sys_call_table[__NR_ptrace];
  sys_call_table[__NR_ptrace]=skas_sys_ptrace;

  printk("SKAS module installed\n");

  return 0;
}

void cleanup_module(void)
{
  /* Return the system call back to normal */
  if (sys_call_table[__NR_ptrace] != skas_sys_ptrace) {
    printk("SKAS - Seperate Kernel Address Space Module\n"
	   "Somebody else has modified the system ptrace\n"
	   "call.  SKAS is returning it to the original\n"
	   "ptrace that comes with your kernel.\n");
  }
  sys_call_table[__NR_ptrace]=sys_ptrace;

  /* get rid of /proc/mm */
  remove_proc_mm();

  printk("SKAS module uninstalled\n");
}

