/* $Id: ptrace.c,v 1.1 2002/12/11 09:14:51 rogerb Exp $ */
#define __KERNEL__
#define MODULE
#include <linux/modversions.h>
#include <linux/module.h>

#include <asm/processor.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/smp_lock.h>

#include <asm/uaccess.h>



#include "skasptrace.h"


extern struct mm_struct *proc_mm_get_mm(int fd);
extern int skas_modify_ldt(struct task_struct *task, int func, void *ptr, unsigned long bytecount);

int skas_ptrace(long request, long pid, long addr, long data)
{
	struct task_struct *child;
	int ret;

	lock_kernel();

	ret = -ESRCH;
	read_lock(&tasklist_lock);
	child = find_task_by_pid(pid);
	if (child)
		get_task_struct(child);
	read_unlock(&tasklist_lock);
	if (!child)
		goto out;

	ret = -EPERM;
	if (pid == 1)		/* you may not mess with init */
		goto out_tsk;

	ret = ptrace_check_attach(child, request == PTRACE_KILL);
	if (ret < 0)
		goto out_tsk;

	switch(request)
	{
	case PTRACE_FAULTINFO: {
		struct ptrace_faultinfo fault;

		fault = ((struct ptrace_faultinfo) 
			{ .is_write	= child->thread.error_code,
			  .addr		= child->thread.cr2 });
		ret = copy_to_user((unsigned long *) data, &fault, 
				   sizeof(fault));
		if(ret)
			break;
		break;
	}
	case PTRACE_SIGPENDING:
		ret = copy_to_user((unsigned long *) data, 
				   &child->pending.signal,
				   sizeof(child->pending.signal));
		break;

	case PTRACE_LDT: {
		struct ptrace_ldt ldt;

		if(copy_from_user(&ldt, (unsigned long *) data, 
				  sizeof(ldt))){
			ret = -EIO;
			break;
		}
		ret = skas_modify_ldt(child, ldt.func, ldt.ptr, ldt.bytecount);
		break;
	}

	case PTRACE_SWITCH_MM: {
		struct mm_struct *old = child->mm;
		struct mm_struct *new = proc_mm_get_mm(data);

		if(IS_ERR(new)){
			ret = PTR_ERR(new);
			break;
		}

		atomic_inc(&new->mm_users);
		child->mm = new;
		child->active_mm = new;
		mmput(old);
		ret = 0;
		break;
	}
		default:
		ret = -EIO;
		break;

	} /* end switch(request) */

out_tsk:
	free_task_struct(child);
out:
	unlock_kernel();
	return ret;
}

	
asmlinkage int skas_sys_ptrace(long request, long pid, long addr, long data)
{
	/* is the request one specific to skas? */
	if (request!=PTRACE_FAULTINFO &&
	    request!=PTRACE_SIGPENDING &&
	    request!=PTRACE_LDT &&
	    request!=PTRACE_SWITCH_MM)
		return (*chained_sys_ptrace)(request,pid,addr,data);
	return skas_ptrace(request,pid,addr,data);
}


/*
 * Overrides for Emacs so that we follow Linus's tabbing style.
 * Emacs will notice this stuff at the end of the file and automatically
 * adjust the settings for this buffer only.  This must remain at the end
 * of the file.
 * ---------------------------------------------------------------------------
 * Local variables:
 * c-file-style: "linux"
 * End:
 */
