#!/bin/sh

# Transmogrifies ldt.c into one that helps SKAS.

# The following changes are made:
#
# Prefix with stuff normal for a module
# 
#  read_ldt() / write_ldt()
#  - prefix with skas_(not strictly necessary since they are static
#    but more of a cosmetic thing)
#  - add first argument of struct task_struct *task
#  - change *mm to point to task -> mm
#
#  sys_modify_ldt()
#  - rename to skas_modify_ldt
#  - remove asmlinkage
#  - add task_struct* task as first parameter
#  - change calls to read/write_ldt to prefix them with skas_ 
#  - add in first parameter of task in calls to above
#

cat <<EOF
#define __KERNEL__
#define MODULE
#include <linux/modversions.h>
#include <linux/module.h>


EOF

# Real men use Perl, I use sed :-)

sed 's/\(static int read_ldt.\)void/\1struct task_struct *task, void/'  | \
sed 's/\(static int write_ldt.\)void/\1struct task_struct *task, void/' | \
sed 's/current->mm/task->mm/'                                           | \
sed 's/= read_ldt(/= read_ldt(task, /'                                  | \
sed 's/= write_ldt(/= write_ldt(task, /'                                | \
sed 's/sys_modify_ldt(/sys_modify_ldt(struct task_struct *task, /'      | \
sed 's/read_ldt/skas_read_ldt/'                                         | \
sed 's/write_ldt/skas_write_ldt/'                                       | \
sed 's/sys_modify_ldt/skas_modify_ldt/'                                 | \
sed 's/asmlinkage //'                                                   
